@@ -1,10 +1,11 @@
 #!/usr/bin/env python3
 
-from typing import List, NamedTuple, Optional, Tuple, Union
+from typing import Any, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import pandas.testing as pdtest
+from pandas.api.types import is_datetime64_dtype
 from sklearn.base import TransformerMixin
 from sklearn.exceptions import NotFittedError
 from sklearn.utils.validation import check_array, check_is_fitted
@@ -21,16 +22,6 @@ TimePredictType = TSCDataFrame
 InitialConditionType = Union[TSCDataFrame, np.ndarray]
 
 
-class FeatureInfo(NamedTuple):
-    quantity: int
-    names: Optional[pd.Index] = None
-
-
-class TimeValueInfo(NamedTuple):
-    quantity: int
-    values: np.ndarray
-
-
 class TSCBaseMixin(object):
     """Base class for Mixin's in *datafold*.
 
@@ -44,9 +35,25 @@ class TSCBaseMixin(object):
         # True, for pandas.DataFrame or TSCDataFrame
         return isinstance(_obj, pd.DataFrame)
 
+    def _read_fit_params(self, attrs: Optional[List[Tuple[str, Any]]], fit_params):
+        return_values = []
+
+        if attrs is not None:
+            for attr in attrs:
+                return_values.append(fit_params.pop(attr[0], attr[1]))
+
+        if fit_params != {}:
+            raise KeyError(f"fit_params.keys = {fit_params.keys()} not supported")
+
+        if len(return_values) == 0:
+            return None
+        elif len(return_values) == 1:
+            return return_values[0]
+        else:
+            return return_values
+
     def _X_to_numpy(self, X):
-        """ Returns a numpy array of the data.
-        """
+        """Returns a numpy array of the data."""
         if self._has_feature_names(X):
             X = X.to_numpy()
             # a row in a df is always a single sample (which requires to be
@@ -58,7 +65,8 @@ class TSCBaseMixin(object):
     def _check_attributes_set_up(self, check_attributes):
         try:
             check_is_fitted(
-                self, attributes=check_attributes,
+                self,
+                attributes=check_attributes,
             )
         except NotFittedError:
             raise RuntimeError(
@@ -66,19 +74,35 @@ class TSCBaseMixin(object):
                 f"Please report bug."
             )
 
-    def _validate_data(
+    def _validate_datafold_data(
         self,
         X: Union[TSCDataFrame, np.ndarray],
         ensure_tsc: bool = False,
-        validate_array_kwargs: Optional[dict] = None,
-        validate_tsc_kwargs: Optional[dict] = None,
+        array_kwargs: Optional[dict] = None,
+        tsc_kwargs: Optional[dict] = None,
     ):
-        """Provides a general function to validate data -- can be overwritten if a
-        concrete implementation requires different checks."""
+        """Provides a general function to validate data that is input to datafold
+        functions -- it can be overwritten if a concrete implementation requires
+        different checks.
+
+        This function is very close to scikit-learn BaseEstimator._validate_data (which
+        was introduced in  0.23.1).
+
+        Parameters
+        ----------
+        X
+        ensure_feature_name_type
+        array_kwargs
+        tsc_kwargs
+
+        Returns
+        -------
+
+        """
 
         # defaults to empty dictionary if None
-        validate_array_kwargs = validate_array_kwargs or {}
-        validate_tsc_kwargs = validate_tsc_kwargs or {}
+        array_kwargs = array_kwargs or {}
+        tsc_kwargs = tsc_kwargs or {}
 
         if ensure_tsc and not isinstance(X, TSCDataFrame):
             raise TypeError(
@@ -94,7 +118,7 @@ class TSCBaseMixin(object):
             #  * pandas.DataFrame (Note a TSCDataFrame is also a pandas.DataFrame,
             #                      but not strictly)
 
-            validate_tsc_kwargs = {}  # no need to check -> overwrite to empty dict
+            tsc_kwargs = {}  # no need to check -> overwrite to empty dict
 
             if type(X) == pd.DataFrame:
                 # special handling of pandas.DataFrame (strictly, not including
@@ -108,18 +132,16 @@ class TSCBaseMixin(object):
 
             X = check_array(
                 X,
-                accept_sparse=validate_array_kwargs.pop("accept_sparse", False),
-                accept_large_sparse=validate_array_kwargs.pop(
-                    "accept_large_sparse", False
-                ),
-                dtype=validate_array_kwargs.pop("dtype", "numeric"),
-                order=validate_array_kwargs.pop("order", None),
-                copy=validate_array_kwargs.pop("copy", False),
-                force_all_finite=validate_array_kwargs.pop("force_all_finite", True),
-                ensure_2d=validate_array_kwargs.pop("ensure_2d", True),
-                allow_nd=validate_array_kwargs.pop("allow_nd", False),
-                ensure_min_samples=validate_array_kwargs.pop("ensure_min_samples", 1),
-                ensure_min_features=validate_array_kwargs.pop("ensure_min_features", 1),
+                accept_sparse=array_kwargs.pop("accept_sparse", False),
+                accept_large_sparse=array_kwargs.pop("accept_large_sparse", False),
+                dtype=array_kwargs.pop("dtype", "numeric"),
+                order=array_kwargs.pop("order", None),
+                copy=array_kwargs.pop("copy", False),
+                force_all_finite=array_kwargs.pop("force_all_finite", True),
+                ensure_2d=array_kwargs.pop("ensure_2d", True),
+                allow_nd=array_kwargs.pop("allow_nd", False),
+                ensure_min_samples=array_kwargs.pop("ensure_min_samples", 1),
+                ensure_min_features=array_kwargs.pop("ensure_min_features", 1),
                 estimator=self,
             )
 
@@ -128,39 +150,32 @@ class TSCBaseMixin(object):
 
         else:
 
-            validate_array_kwargs = {}  # no need to check -> overwrite to empty dict
+            array_kwargs = {}  # no need to check -> overwrite to empty dict
 
             X = X.tsc.check_tsc(
-                ensure_all_finite=validate_tsc_kwargs.pop("ensure_all_finite", True),
-                ensure_same_length=validate_tsc_kwargs.pop("ensure_same_length", False),
-                ensure_const_delta_time=validate_tsc_kwargs.pop(
+                ensure_all_finite=tsc_kwargs.pop("ensure_all_finite", True),
+                ensure_min_samples=tsc_kwargs.pop("ensure_min_samples", None),
+                ensure_same_length=tsc_kwargs.pop("ensure_same_length", False),
+                ensure_const_delta_time=tsc_kwargs.pop(
                     "ensure_const_delta_time", False
                 ),
-                ensure_delta_time=validate_tsc_kwargs.pop("ensure_delta_time", None),
-                ensure_same_time_values=validate_tsc_kwargs.pop(
+                ensure_delta_time=tsc_kwargs.pop("ensure_delta_time", None),
+                ensure_same_time_values=tsc_kwargs.pop(
                     "ensure_same_time_values", False
                 ),
-                ensure_normalized_time=validate_tsc_kwargs.pop(
-                    "ensure_normalized_time", False
-                ),
-                ensure_n_timeseries=validate_tsc_kwargs.pop(
-                    "ensure_n_timeseries", None
-                ),
-                ensure_min_timesteps=validate_tsc_kwargs.pop(
-                    "ensure_min_timesteps", None
-                ),
-                ensure_no_degenerate_ts=validate_tsc_kwargs.pop(
+                ensure_normalized_time=tsc_kwargs.pop("ensure_normalized_time", False),
+                ensure_n_timeseries=tsc_kwargs.pop("ensure_n_timeseries", None),
+                ensure_min_timesteps=tsc_kwargs.pop("ensure_min_timesteps", None),
+                ensure_no_degenerate_ts=tsc_kwargs.pop(
                     "ensure_no_degenerate_ts", False
                 ),
             )
 
-        if validate_array_kwargs != {} or validate_tsc_kwargs != {}:
+        if array_kwargs != {} or tsc_kwargs != {}:
             # validate_kwargs have to be empty and must only contain key-values that can
             # be handled to check_array / check_tsc
 
-            left_over_keys = list(validate_array_kwargs.keys()) + list(
-                validate_tsc_kwargs.keys()
-            )
+            left_over_keys = list(array_kwargs.keys()) + list(tsc_kwargs.keys())
             raise ValueError(
                 f"{left_over_keys} are no valid validation keys. Please report bug."
             )
@@ -177,47 +192,47 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
     * :class:`pandas.DataFrame` no restriction on the frame's index and column format
     * :class:`.TSCDataFrame` as a special data frame for time series collections
 
+    The parameters should be set in during `fit` in a subclass.
+
+    .. note::
+
+        The scikit-learn project heavily discusses on how to handle feature names. There
+        are many proposed solutions. The solution that datafold uses is
+        `SLEP007 <https://scikit-learn-enhancement-proposals.readthedocs.io/en/latest/slep007/proposal.html>`__
+
+        However, this is only a proposal and may have to be changed later.
+
+        Other resources:
+
+        * `new array (SLEP012) <https://scikit-learn-enhancement-proposals.readthedocs.io/en/latest/slep012/proposal.html>`__
+        * `discussion (SLEP008) <https://github.com/scikit-learn/enhancement_proposals/pull/18/>`__
+
     Parameters
     ----------
 
-    features_in_: FeatureInfo
-        Number of features during fit and corresponding feature names. The attribute
-        should be set in during `fit`. Set feature names in `inverse_transform` and
-        validate input in `transform`.
+    n_features_in_: int
+        Number of features of input during `fit`.
 
-    features_out_: FeatureInfo
-        Number of features and corresponding feature names after transformation.
-        The attribute should be set in during `fit`. Set feature names in
-        `transform` and validate input in `inverse_transform`.
+    n_features_in_: Optional[pd.Index]
+        Feature names during `fit` if the input is indexed. The feature names
+        are used as output in `inverse_transform` and for validation in `transform`.
+
+    n_features_out_: int
+        Number of features of output during `fit`.
+
+    features_out_: Optional[pd.Index]
+        Feature names during `fit` if the input is indexed. The feature names
+        are used as output in `transform` and for validation in `inverse_transform`.
     """
 
-    _feature_attrs = ["features_in_", "features_out_"]
+    _feature_attrs = [
+        "n_features_in_",
+        "n_features_out_",
+        "feature_names_in_",
+        "feature_names_out_",
+    ]
 
-    def _setup_frame_input_fit(self, features_in: pd.Index, features_out: pd.Index):
-
-        if features_in.has_duplicates or features_out.has_duplicates:
-            raise ValueError(
-                "duplicated indices detected. \n"
-                f"features_in={features_in.duplicated()} \n"
-                f"features_out={features_out.duplicated()}"
-            )
-
-        if features_in.ndim != 1 or features_out.ndim != 1:
-            raise ValueError("feature names must be 1-dim.")
-
-        self.features_in_: FeatureInfo = FeatureInfo(
-            quantity=len(features_in), names=features_in
-        )
-        self.features_out_: FeatureInfo = FeatureInfo(
-            quantity=len(features_out), names=features_out
-        )
-
-    def _setup_array_input_fit(self, features_in: int, features_out: int):
-        # do not store names, because they are not available
-        self.features_in_ = FeatureInfo(quantity=features_in)
-        self.features_out_ = FeatureInfo(quantity=features_out)
-
-    def _setup_features_fit(self, X, features_out):
+    def _setup_feature_attrs_fit(self, X, features_out):
 
         if isinstance(features_out, str):
             assert features_out == "like_features_in"
@@ -231,12 +246,26 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
                 # For convenience features_out can be given as a list
                 # (better code readability than pd.Index)
                 features_out = pd.Index(
-                    features_out, dtype=np.str, name=TSCDataFrame.tsc_feature_col_name,
+                    features_out,
+                    dtype=np.str_,
+                    name=TSCDataFrame.tsc_feature_col_name,
                 )
 
-            self._setup_frame_input_fit(
-                features_in=X.columns, features_out=features_out
-            )
+            if X.columns.has_duplicates or features_out.has_duplicates:
+                raise ValueError(
+                    "duplicated indices detected. \n"
+                    f"features_in={X.columns.duplicated()} \n"
+                    f"features_out={features_out.duplicated()}"
+                )
+
+            if X.columns.ndim != 1 or features_out.ndim != 1:
+                raise ValueError("feature names must be 1-dim.")
+
+            self.n_features_in_: int = len(X.columns)
+            self.n_features_out_: int = len(features_out)
+            self.feature_names_in_: Optional[pd.Index] = X.columns
+            self.feature_names_out_: Optional[pd.Index] = features_out
+
         else:
 
             if isinstance(features_out, str) and features_out == "like_features_in":
@@ -247,34 +276,40 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
                 # if list or pd.Index use the number of features out
                 features_out = len(features_out)
 
-            self._setup_array_input_fit(
-                features_in=X.shape[1], features_out=features_out
-            )
+            # do not store names, because they are not available
+            self.n_features_in_ = X.shape[1]
+            self.n_features_out_ = features_out
+            self.feature_names_in_ = None
+            self.feature_names_out_ = None
 
     def _validate_feature_input(self, X: TransformType, direction):
 
         self._check_attributes_set_up(self._feature_attrs)
 
-        if not self._has_feature_names(X) or self.features_out_.names is None:
+        if not self._has_feature_names(X) or self.feature_names_out_ is None:
             # Either
             # * X has no feature names, or
             # * during fit X had no feature names given.
             # --> Only check if shape is correct and trust user with the rest
-            if self.features_in_.quantity != X.shape[1]:
+
+            if direction == "transform":
+                _check_shape = self.n_features_in_
+            else:  # direction == "inverse_transform"
+                _check_shape = self.n_features_out_
+
+            if _check_shape != X.shape[1]:
                 raise ValueError(
-                    f"Shape mismatch: expected {self.features_out_.quantity} "
+                    f"Shape mismatch: expected {self.n_features_out_} "
                     f"features (number of columns in 'X') but got {X.shape[1]}."
                 )
         else:  # self._has_feature_names(X)
             # Now X has features and during fit features were given. So now we can
             # check if feature names of X match with data during fit:
 
-            _check_features: Tuple[int, pd.Index]
-
             if direction == "transform":
-                _check_features = self.features_in_
+                _check_features = self.feature_names_in_
             elif direction == "inverse_transform":
-                _check_features = self.features_out_
+                _check_features = self.feature_names_out_
             else:
                 raise RuntimeError(
                     f"'direction'={direction} not known. Please report bug."
@@ -283,13 +318,13 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
             if isinstance(X, pd.Series):
                 # if X is a Series, then the columns of the original data are in a
                 # Series this usually happens if X.iloc[0, :] --> returns a Series
-                pdtest.assert_index_equal(right=_check_features[1], left=X.index)
+                pdtest.assert_index_equal(right=_check_features, left=X.index)
             else:
-                pdtest.assert_index_equal(right=_check_features[1], left=X.columns)
+                pdtest.assert_index_equal(right=_check_features, left=X.columns)
 
     def _same_type_X(
         self, X: TransformType, values: np.ndarray, feature_names: pd.Index
-    ) -> TransformType:
+    ) -> Union[pd.DataFrame, TransformType]:
         """Chooses the same type for input as type of `X`.
 
         Parameters
@@ -342,7 +377,7 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
         y : None
             ignored
 
-        **fit_params : dict
+        **fit_params: Dict[str, object]
             Additional fit parameters.
 
         Returns
@@ -358,49 +393,57 @@ class TSCTransformerMixin(TSCBaseMixin, TransformerMixin):
 class TSCPredictMixin(TSCBaseMixin):
     """Mixin to provide functionality for models that train on time series data.
 
+    The attribute should be set during `fit` and used to validate during `predict`.
+
     Parameters
     ----------
 
-    features_in_: Tuple[int, pandas.Index]
-        Number of features during fit and corresponding feature names. The attribute
-        should be set during `fit` and used to validate during `predict`.
+    n_features_in_: int
+        Number of features during `fit`.
 
-    time_values_in_: Tuple[int, numpy.ndarray]
-        Number of time values and array with all time values during `fit`. Note
-        because in a time seres collection not all time series share the same time
-        samples, a time value has to appear in at least one time series to be listed in
-        `time_values_in_`. The attribute should be set during `fit`, can be used for
-        default time values and allows reasonable prediction intervals to be validated.
+    feature_names_in_: pd.Index
+        The feature names during `fit`.
 
-    dt_
-        Time sampling in the data during fit.
+    time_values_in_: numpy.ndarray
+        Time values with all time values observed during `fit`. Note, that because in a
+        time series collection not all time series must share the same time
+        values, a time value to be recorded in `time_values_in_` must at least appear
+        in one time series.
+
+    dt_: Union[float, pd.Series]
+        Time sampling rate in the time series data during `fit`.
     """
 
-    _cls_feature_attrs = ["features_in_", "time_values_in_", "dt_"]
+    _cls_feature_attrs = [
+        "n_features_in_",
+        "feature_names_in_",
+        "time_values_in_",
+        "dt_",
+    ]
 
     @property
     def time_interval_(self):
         self._check_attributes_set_up(check_attributes="time_values_in_")
-        return (self.time_values_in_.values[0], self.time_values_in_.values[-1])
+        return (self.time_values_in_[0], self.time_values_in_[-1])
 
     def _setup_default_tsc_metric_and_score(self):
         self.metric_eval = TSCMetric(metric="rmse", mode="feature", scaling="min-max")
         self._score_eval = TSCScoring(self.metric_eval)
 
-    def _setup_features_and_time_fit(self, X: TSCDataFrame):
+    def _setup_features_and_time_attrs_fit(self, X: TSCDataFrame):
 
         if not isinstance(X, TSCDataFrame):
-            raise TypeError("Only TSCDataFrame can be used for 'X'. ")
+            raise TypeError("Only TSCDataFrame can be used for 'X'.")
 
         time_values = X.time_values()
         features_in = X.columns
 
         time_values = self._validate_time_values(time_values=time_values)
-        self.time_values_in_: TimeValueInfo = TimeValueInfo(
-            len(time_values), time_values
-        )
-
+        self.time_values_in_ = time_values
+        self.n_features_in_ = len(features_in)
+        self.feature_names_in_ = features_in
         self.dt_ = X.delta_time
+
         if isinstance(self.dt_, pd.Series) or np.isnan(
             self.dt_
         ):  # Series if dt_ is not the same across multiple time series.
@@ -418,10 +461,6 @@ class TSCPredictMixin(TSCBaseMixin):
             == 0
         )
 
-        self.features_in_: FeatureInfo = FeatureInfo(
-            quantity=len(features_in), names=features_in
-        )
-
     def _validate_time_values(self, time_values: np.ndarray):
 
         try:
@@ -437,20 +476,21 @@ class TSCPredictMixin(TSCBaseMixin):
             # https://docs.scipy.org/doc/numpy/reference/generated/numpy.dtype.kind.html
             raise TypeError(f"time_values.dtype {time_values.dtype} not supported")
 
-        _is_datetime = time_values.dtype.kind == "M"
+        if not is_datetime64_dtype(time_values) and (time_values < 0).any():
+            # "datetime" cannot be negative and raises error when checked with "< 0"
+            raise ValueError("In 'time_values' all values must be non-negative.")
 
-        if not _is_datetime and (time_values < 0).any():
-            # "datetime" cannot be negative and cannot be checked with "< 0"
-            raise ValueError("in time_values no negative values are allowed")
+        if not np.isfinite(time_values).all():
+            raise ValueError("'time_values' contains invalid numbers (inf/nan).")
 
         if time_values.ndim != 1:
-            raise ValueError("time_values must be be one dimensional")
+            raise ValueError("'time_values' must be be an 1-dim. array")
 
-        if not (np.diff(time_values).astype(np.float64) > 0).all():
+        if not (np.diff(time_values).astype(float) > 0).all():
             # as "float64" is required in case of datetime where the differences are in
             # terms of "np.timedelta"
             raise ValueError(
-                "Parameter 'time_values' must be sorted with increasing unique values."
+                "'time_values' must be sorted with increasing unique values"
             )
 
         return time_values
@@ -469,22 +509,45 @@ class TSCPredictMixin(TSCBaseMixin):
             )
 
     def _validate_feature_names(self, X: TransformType, require_all=True):
-        self._check_attributes_set_up(check_attributes=["features_in_"])
+        self._check_attributes_set_up(check_attributes=["feature_names_in_"])
 
         try:
             if require_all:
                 pdtest.assert_index_equal(
-                    right=self.features_in_.names, left=X.columns, check_names=False
+                    right=self.feature_names_in_, left=X.columns, check_names=False
                 )
             else:
-                if not np.isin(X.columns, self.features_in_.names).all():
+                if not np.isin(X.columns, self.feature_names_in_).all():
                     raise AssertionError(
                         f"feature names in X are invalid "
-                        f"{X.columns[np.isin(self.features_in_.names,X.columns)]}"
+                        f"{X.columns[np.isin(self.feature_names_in_,X.columns)]}"
                     )
         except AssertionError as e:
             raise ValueError(e.args[0])
 
+    def _validate_qois(self, qois, valid_feature_names) -> np.ndarray:
+
+        if qois is not None:
+            try:
+                qois = np.asarray(qois)
+            except:
+                raise TypeError("parameter 'qois' must be list-like")
+
+            if qois.ndim != 1:
+                raise ValueError(
+                    f"'qois' must be a 1-dim. array. " f"Got qois.ndim={qois.ndim}"
+                )
+
+            mask_valid_qois = np.isin(qois, valid_feature_names)
+
+            if not mask_valid_qois.all():
+                raise ValueError(
+                    f"The qois={qois[~mask_valid_qois]} are invalid. Valid "
+                    f"feature names are {valid_feature_names}."
+                )
+
+        return qois
+
     def _validate_features_and_time_values(
         self, X: TSCDataFrame, time_values: Optional[np.ndarray]
     ):
@@ -492,12 +555,12 @@ class TSCPredictMixin(TSCBaseMixin):
         self._check_attributes_set_up(check_attributes=["time_values_in_"])
 
         if time_values is None:
-            time_values = self.time_values_in_.values
+            time_values = self.time_values_in_
 
         if not self._has_feature_names(X):
             raise TypeError("only types that support feature names are supported")
 
-        self._validate_time_values(time_values=time_values)
+        time_values = self._validate_time_values(time_values=time_values)
         self._validate_feature_names(X)
 
         return X, time_values
@@ -505,13 +568,6 @@ class TSCPredictMixin(TSCBaseMixin):
     def fit(self, X: TimePredictType, **fit_params):
         raise NotImplementedError("method not implemented")
 
-    def reconstruct(
-        self,
-        X: TSCDataFrame,
-        qois: Optional[Union[np.ndarray, pd.Index, List[str]]] = None,
-    ):
-        raise NotImplementedError("method not implemented")
-
     def predict(
         self,
         X: InitialConditionType,
@@ -519,3 +575,10 @@ class TSCPredictMixin(TSCBaseMixin):
         **predict_params,
     ):
         raise NotImplementedError("method not implemented")
+
+    def reconstruct(
+        self,
+        X: TSCDataFrame,
+        qois: Optional[Union[np.ndarray, pd.Index, List[str]]] = None,
+    ):
+        raise NotImplementedError("method not implemented")
