@@ -1,4 +1,5 @@
 import sys
+import warnings
 from typing import List, Optional, Tuple, Union
 
 import numpy as np
@@ -8,6 +9,7 @@ import scipy.sparse
 import scipy.sparse.linalg
 import scipy.spatial
 from sklearn.base import BaseEstimator
+from sklearn.utils import resample
 from sklearn.utils.validation import check_is_fitted, check_scalar
 
 from datafold.dynfold.base import TransformType, TSCTransformerMixin
@@ -19,6 +21,7 @@ from datafold.pcfold.kernels import (
     GaussianKernel,
     KernelType,
     PCManifoldKernel,
+    TSCManifoldKernel,
 )
 from datafold.utils.general import (
     df_type_and_indices_from,
@@ -52,13 +55,11 @@ class _DmapKernelAlgorithms:
     ) -> Tuple[np.ndarray, Union[np.ndarray, TSCDataFrame]]:
 
         if isinstance(kernel_matrix, pd.DataFrame):
-            _kernel_matrix = kernel_matrix.to_numpy()
-        else:
-            _kernel_matrix = kernel_matrix
+            kernel_matrix = kernel_matrix.to_numpy()
 
         try:
             eigvals, eigvect = compute_kernel_eigenpairs(
-                kernel_matrix=_kernel_matrix,  # from here on only ndarray
+                kernel_matrix=kernel_matrix,  # from here on only ndarray
                 n_eigenpairs=n_eigenpairs,
                 is_symmetric=is_symmetric,
                 is_stochastic=is_stochastic,
@@ -82,7 +83,7 @@ class _DmapKernelAlgorithms:
         if raise_numerical_error:
             raise NumericalMathError(
                 "Eigenpairs have non-negligible imaginary part (larger than "
-                f"{1e2 * sys.float_info.epsilon}. First try to use "
+                f"{1e2 * sys.float_info.epsilon}). First try to use "
                 f"parameter 'symmetrize_kernel=True' (improves numerical stability) and "
                 f"only if this is not working adjust kernel settings."
             )
@@ -106,18 +107,35 @@ class _DmapKernelAlgorithms:
 
     @staticmethod
     def unsymmetric_kernel_matrix(
-        kernel_matrix: KernelType, basis_change_matrix,
+        kernel_matrix: KernelType,
+        basis_change_matrix,
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
         """Transform a kernel matrix obtained from a symmetric conjugate
         transformation to the diffusion kernel matrix.
 
+        The conjugate relationship is as follows
+
+        .. math::
+            A = D^{1/2} K D^{-1/2}
+
+        , where :math:`A` is the symmetric matrix, conjugate to the "true" Markov
+        matrix :math:`K`. To recover :math:`K` the following operation is performed
+
+        .. math::
+            K = D^{-1/2} A D^{1/2}
+
+        Note, that the ``basis_change_matrix``, is already :math:`D^{-1/2}`.
+        See also reference :cite:`rabin_heterogeneous_2012` and function
+        :py:meth:`_conjugate_stochastic_kernel_matrix` in ``kernels.py`` for further
+        infos.
+
         Parameters
         ----------
         kernel_matrix
             Symmetric conjugate kernel matrix of shape `(n_samples, n_samples)`
 
         basis_change_matrix
-            diagonal elements of basis change matrix
+            Diagonal elements of basis change matrix :math:`D^{-1/2}`
 
         Returns
         -------
@@ -134,9 +152,7 @@ class _DmapKernelAlgorithms:
             np.reciprocal(basis_change_matrix.data.ravel())
         )
 
-        kernel_matrix = (
-            inv_basis_change_matrix @ kernel_matrix @ inv_basis_change_matrix
-        )
+        kernel_matrix = basis_change_matrix @ kernel_matrix @ inv_basis_change_matrix
 
         if row_idx is not None and col_idx is not None:
             kernel_matrix = TSCDataFrame(kernel_matrix, index=row_idx, columns=col_idx)
@@ -145,12 +161,13 @@ class _DmapKernelAlgorithms:
 
 
 class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
-    """Define diffusion process on point cloud to find meaningful geometric descriptions.
+    """Define a diffusion process on point cloud to find meaningful
+    geometric descriptions.
 
     The model can be used for
 
-    * non-linear dimensionality reduction
-    * approximating eigenfunctions of operators (see ``alpha`` parameter):
+    * non-linear dimension reduction
+    * approximation eigenfunctions of operators (see ``alpha`` parameter):
 
         - Laplace-Beltrami
         - Fokker-Plank
@@ -159,37 +176,38 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
     Parameters
     ----------
     kernel
-        The kernel to describe proximity between points. The kernel is passed
-        as an ``internal_kernel`` to :py:class:`.DmapKernelFixed`, which describes
-        the diffusion process.
+        The kernel to describe similarity between points. The kernel is passed
+        as an ``internal_kernel`` to :py:class:`.DmapKernelFixed`, which computes a
+        Markov matrix that describes the diffusion process.
+        Defaults to :py:class:`.GaussianKernel` with bandwidth `epsilon=1`.
 
     n_eigenpairs
-        The number of eigenpairs to compute from kernel matrix.
+        The number of eigenpairs to compute from the kernel matrix.
 
     time_exponent
-        The time of the diffusion process (exponent of eigenvalues in embedding).
+        The time of the diffusion process (exponent of eigenvalues in embedding). The
+        value can be changed after the model is fit.
 
     is_stochastic
         If True, the diffusion kernel matrix is normalized (row stochastic). In the
-        standard definition of diffusion maps this has to be true.
+        standard definition this has to be True.
 
     alpha
-        The re-normalization parameter between `(0,1)`. Set ``alpha=1`` to correct the
-        sampling density in the data as an artifact of the collection process. Special
-        values are
+        The degree of re-normalization between `(0,1)`. Setting ``alpha=1`` corrects
+        the sampling density in the data as an artifact of the collection process.
+        Special values are (see :cite:`coifman_diffusion_2006`)
 
         * `alpha=0` Graph Laplacian,
         * `alpha=0.5` Fokker-Plank operator,
         * `alpha=1` Laplace-Beltrami operator
 
-        Note, that ``is_stochastic=True`` is required in all three cases.
+        Note, that ``is_stochastic=True`` is required for all three cases.
 
     symmetrize_kernel
-        If True, a conjugate transformation is performed if the settings
-        lead to a non-symmetric kernel matrix. This improves numerical stability when
-        solving the eigenvectors of the kernel matrix because it allows algorithms
-        designed for (sparse) Hermitian matrices to be used. If the kernel matrix is
-        symmetric already (`is_stochastic=False`), then the parameter has no effect.
+        If True, a symmetric conjugate transformation is performed, if the kernel
+        matrix is non-symmetric (otherwise the parameter has no effect). The symmetric
+        conjugate improves numerical stability when solving the eigenvectors, because it
+        allows using algorithms designed for (sparse) Hermitian matrices.
 
     dist_kwargs
         Keyword arguments passed to the internal distance matrix computation. See
@@ -199,20 +217,27 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
     Attributes
     ----------
 
-    X_: PCManifold
+    X_fit_: PCManifold
         The training data during fit. The data is required for out-of-sample mappings;
-        the object is equipped with kernel :py:class:`DmapKernelFixed`.
+        the :py:class:`PCManifold` is equipped with kernel :py:class:`DmapKernelFixed`.
+        ``np.asarray(X_fit_)`` casts the object to a standard numpy array.
 
     eigenvalues_ : numpy.ndarray
         The eigenvalues of diffusion kernel matrix in decreasing order.
 
     eigenvectors_: TSCDataFrame, pandas.DataFrame, numpy.ndarray
-        The eigenvectors of the kernel matrix to parametrize the data manifold.
+        The eigenvectors of the kernel matrix.
+
+    target_coords_: numpy.ndarray
+        The coordinate indices to map to when transforming the data. The target point
+        dimension equals the number of indices included in `target_coords_`. Note that the
+        attributes `eigenvectors_` and `eigenvalues_` sill contain *all* computed
+        eigenpairs.
 
     inv_coeff_matrix_: numpy.ndarray
-        The coefficient matrix to map points from embedding space back to original space.\
-        The computation and setting the attribute is delayed until `inverse_transform` is
-        called for the first time.
+        The coefficient matrix to map points from embedding space back to state space.
+        The computation is delayed until `inverse_transform` is called for the first
+        time (only then the attribute is available).
 
     kernel_matrix_ : Union[numpy.ndarray, scipy.sparse.csr_matrix]
         The computed kernel matrix; the matrix is only stored if
@@ -233,7 +258,8 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
 
     def __init__(
         self,
-        kernel: BaseManifoldKernel = GaussianKernel(epsilon=1.0),
+        kernel: Optional[Union[PCManifoldKernel, TSCManifoldKernel]] = None,
+        *,  # keyword-only
         n_eigenpairs: int = 10,
         time_exponent: float = 0,
         is_stochastic: bool = True,
@@ -256,7 +282,10 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
 
     @classmethod
     def laplace_beltrami(
-        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
+        cls,
+        kernel=GaussianKernel(epsilon=1.0),
+        n_eigenpairs=10,
+        **kwargs,
     ) -> "DiffusionMaps":
         """Instantiate new model to approximate Laplace-Beltrami operator.
 
@@ -275,7 +304,10 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
 
     @classmethod
     def fokker_planck(
-        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
+        cls,
+        kernel=GaussianKernel(epsilon=1.0),
+        n_eigenpairs=10,
+        **kwargs,
     ) -> "DiffusionMaps":
         """Instantiate new model to approximate Fokker-Planck operator.
 
@@ -294,7 +326,10 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
 
     @classmethod
     def graph_laplacian(
-        cls, kernel=GaussianKernel(epsilon=1.0), n_eigenpairs=10, **kwargs,
+        cls,
+        kernel=GaussianKernel(epsilon=1.0),
+        n_eigenpairs=10,
+        **kwargs,
     ) -> "DiffusionMaps":
         """Instantiate new model to approximate graph Laplacian.
 
@@ -311,20 +346,50 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
             **kwargs,
         )
 
-    def _nystrom(self, kernel_cdist, eigvec, eigvals):
+    def _validate_settings(self):
+        check_scalar(
+            self.n_eigenpairs, "n_eigenpairs", target_type=(int, np.integer), min_val=1
+        )
+
+        check_scalar(
+            self.time_exponent,
+            "time_exponent",
+            target_type=(float, int, np.floating, np.integer),
+        )
+
+        check_scalar(
+            self.alpha,
+            "alpha",
+            target_type=(float, np.floating, int, np.integer),
+            min_val=0,
+            max_val=1,
+        )
+
+    def _get_default_kernel(self):
+        return GaussianKernel(epsilon=1.0)
+
+    def _nystrom(self, kernel_cdist, eigvec, eigvals, index_from):
 
         if isinstance(kernel_cdist, pd.DataFrame):
             _kernel_cdist = kernel_cdist.to_numpy()
         else:
             _kernel_cdist = kernel_cdist
 
+        _magic_tol = 1e-14  # need to adapt if not sufficient
+        if (np.abs(eigvals) < _magic_tol).any():
+            warnings.warn(
+                "Diffusion map eigenvalues are close to zero, which can cause "
+                "numerical instabilities when applying the Nystroem extension."
+            )
+
+        # Nystroem approximation
         approx_eigenvectors = _kernel_cdist @ mat_dot_diagmat(
             eigvec, np.reciprocal(eigvals)
         )
 
-        if isinstance(kernel_cdist, pd.DataFrame):
+        if index_from is not None:
             approx_eigenvectors = df_type_and_indices_from(
-                kernel_cdist,
+                index_from,
                 values=approx_eigenvectors,
                 except_columns=[f"ev{i}" for i in range(self.n_eigenpairs)],
             )
@@ -339,25 +404,37 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
             self.time_exponent,
             "time_exponent",
             target_type=(float, np.floating, int, np.integer),
-            min_val=0,
-            max_val=None,
         )
 
         if self.time_exponent == 0:
             dmap_embedding = eigenvectors
         else:
             eigvals_time = np.power(self.eigenvalues_, self.time_exponent)
-            dmap_embedding = diagmat_dot_mat(eigvals_time, np.asarray(eigenvectors))
+            dmap_embedding = mat_dot_diagmat(np.asarray(eigenvectors), eigvals_time)
 
-        if isinstance(eigenvectors, (pd.DataFrame, TSCDataFrame)):
+        if isinstance(eigenvectors, pd.DataFrame):
             dmap_embedding = df_type_and_indices_from(
                 indices_from=eigenvectors,
                 values=dmap_embedding,
-                except_columns=self.features_out_.names,
+                except_columns=self.feature_names_out_,
             )
 
         return dmap_embedding
 
+    def _feature_names(self):
+        if hasattr(self, "target_coords_"):
+            feature_names = pd.Index(
+                [f"dmap{i}" for i in self.target_coords_],
+                name=TSCDataFrame.tsc_feature_col_name,
+            )
+        else:
+            feature_names = pd.Index(
+                [f"dmap{i}" for i in range(self.n_eigenpairs)],
+                name=TSCDataFrame.tsc_feature_col_name,
+            )
+
+        return feature_names
+
     def _setup_default_dist_kwargs(self):
         from copy import deepcopy
 
@@ -366,14 +443,35 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
         self.dist_kwargs_.setdefault("kmin", 0)
         self.dist_kwargs_.setdefault("backend", "guess_optimal")
 
-    def set_coords(self, indices: Union[np.ndarray, List[int]]) -> "DiffusionMaps":
+    def _select_eigenpairs_target_coords(self):
+        """Returns either
+        * all eigenpairs, or
+        * the ones that were selected during set_target_coords
+
+        It is assumed that the model is already fit.
+        """
+
+        if hasattr(self, "target_coords_"):
+            if isinstance(self.eigenvectors_, pd.DataFrame):
+                eigvec = self.eigenvectors_.iloc[:, self.target_coords_]
+            else:
+                eigvec = self.eigenvectors_[:, self.target_coords_]
+
+            eigvals = self.eigenvalues_[self.target_coords_]
+        else:
+            eigvec, eigvals = self.eigenvectors_, self.eigenvalues_
+        return eigvec, eigvals
+
+    def set_target_coords(
+        self, indices: Union[np.ndarray, List[int]]
+    ) -> "DiffusionMaps":
         """Set eigenvector coordinates for parsimonious mapping.
 
         Parameters
         ----------
         indices
-            Index values of ``eigenvalues_`` and respective ``eigenvectors_`` to keep in
-            the model.
+            Index values of eigenpairs (``eigenvalues_`` and ``eigenvectors_``) to map
+            new points to.
 
         Returns
         -------
@@ -381,18 +479,31 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
             self
         """
 
-        check_is_fitted(self, attributes=["eigenvectors_", "eigenvalues_"])
-
         indices = np.asarray(indices)
+        indices = np.sort(indices)
 
-        # type hints for mypy
-        self.eigenvectors_ = if1dim_colvec(self.eigenvectors_[:, indices])
-        self.eigenvalues_ = self.eigenvalues_[indices]
+        if indices.dtype != int:
+            raise TypeError(f"The indices must be integers. Got type {indices.dtype}.")
+
+        if indices[0] < 0 or indices[-1] >= self.n_eigenpairs:
+            raise ValueError(
+                f"Indices {indices} are out of bound. Only integer values "
+                f"in [0, {self.n_eigenpairs}] are allowed."
+            )
+
+        self.target_coords_ = indices
+
+        self.n_features_out_ = len(self.target_coords_)
+        if hasattr(self, "feature_names_out_") and self.feature_names_out_ is not None:
+            self.feature_names_out_ = self._feature_names()
 
         return self
 
     def fit(
-        self, X: TransformType, y=None, store_kernel_matrix=False
+        self,
+        X: TransformType,
+        y=None,
+        **fit_params,
     ) -> "DiffusionMaps":
         """Compute diffusion kernel matrix and its' eigenpairs.
 
@@ -400,45 +511,61 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
             Training data of shape `(n_samples, n_features)`.
-        
+
         y: None
             ignored
 
-        store_kernel_matrix
-            If True, store the kernel matrix in attribute ``kernel_matrix_``.
+        **fit_params: Dict[str, object]
+            - store_kernel_matrix: ``bool``
+                If True, store the kernel matrix in attribute ``kernel_matrix_``.
 
         Returns
         -------
         DiffusionMaps
             self
         """
+        self._validate_settings()
 
-        X = self._validate_data(X=X, validate_array_kwargs=dict(ensure_min_samples=2))
+        X = self._validate_datafold_data(
+            X=X,
+            array_kwargs=dict(ensure_min_samples=max(2, self.n_eigenpairs)),
+            tsc_kwargs=dict(ensure_min_samples=max(2, self.n_eigenpairs)),
+        )
 
-        self._setup_features_fit(
-            X, features_out=[f"dmap{i}" for i in range(self.n_eigenpairs)]
+        self._setup_feature_attrs_fit(X, features_out=self._feature_names())
+
+        store_kernel_matrix = self._read_fit_params(
+            attrs=[("store_kernel_matrix", False)],
+            fit_params=fit_params,
         )
 
         self._setup_default_dist_kwargs()
 
         # Note the DmapKernel is a kernel that wraps another kernel to provides the
         # DMAP specific functionality.
+        internal_kernel = (
+            self.kernel if self.kernel is not None else self._get_default_kernel()
+        )
+
         self._dmap_kernel = DmapKernelFixed(
-            internal_kernel=self.kernel,
+            internal_kernel=internal_kernel,
             is_stochastic=self.is_stochastic,
             alpha=self.alpha,
             symmetrize_kernel=self.symmetrize_kernel,
         )
 
         if isinstance(X, TSCDataFrame):
-            X.kernel = self._dmap_kernel
-            self.X_ = X
+            self.X_fit_ = TSCDataFrame(
+                X, kernel=self._dmap_kernel, dist_kwargs=self.dist_kwargs_
+            )
         elif isinstance(X, (np.ndarray, pd.DataFrame)):
-            self.X_ = PCManifold(
-                X, kernel=self._dmap_kernel, dist_kwargs=self.dist_kwargs_,
+            self.X_fit_ = PCManifold(
+                X,
+                kernel=self._dmap_kernel,
+                dist_kwargs=self.dist_kwargs_,
             )
 
-        kernel_output = self.X_.compute_kernel_matrix()
+        kernel_output = self.X_fit_.compute_kernel_matrix()
         (
             kernel_matrix_,
             self._cdist_kwargs,
@@ -454,13 +581,13 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
             # dynamics-adapted kernels can drop samples from X)
             index_from: Optional[TSCDataFrame] = kernel_matrix_
         elif (
-            isinstance(kernel_matrix_, TSCDataFrame)
-            and kernel_matrix_.shape[0] == self.X_.shape[0]
+            isinstance(self.X_fit_, TSCDataFrame)
+            and kernel_matrix_.shape[0] == self.X_fit_.shape[0]
         ):
-            # if kernel is numpy.ndarray or scipy.sparse.csr_matrix, but X_ is a time
-            # series, then take incides from X_ -- this only works if no samples are
+            # if kernel is numpy.ndarray or scipy.sparse.csr_matrix, but X_fit_ is a time
+            # series, then take incides from X_fit_ -- this only works if no samples are
             # dropped in the kernel computation.
-            index_from = self.X_
+            index_from = self.X_fit_
         else:
             index_from = None
 
@@ -476,9 +603,13 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
             index_from=index_from,
         )
 
+        # TODO: warning if less self.eigenvalues_ than n_eigenpairs?
+        #  happens if square kernel has less entries than n_eigenpairs
+
         if self._dmap_kernel.is_symmetric_transform() and store_kernel_matrix:
             kernel_matrix_ = _DmapKernelAlgorithms.unsymmetric_kernel_matrix(
-                kernel_matrix=kernel_matrix_, basis_change_matrix=basis_change_matrix,
+                kernel_matrix=kernel_matrix_,
+                basis_change_matrix=basis_change_matrix,
             )
 
         if store_kernel_matrix:
@@ -517,20 +648,36 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
             same type as `X` of shape `(n_samples, n_coords)`
         """
-        check_is_fitted(self, ("X_", "eigenvalues_", "eigenvectors_"))
+        check_is_fitted(self, ("X_fit_", "eigenvalues_", "eigenvectors_"))
 
-        X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_samples=1))
+        X = self._validate_datafold_data(X, array_kwargs=dict(ensure_min_samples=1))
         self._validate_feature_input(X, direction="transform")
 
-        kernel_output = self.X_.compute_kernel_matrix(X, **self._cdist_kwargs)
+        kernel_output = self.X_fit_.compute_kernel_matrix(X, **self._cdist_kwargs)
         kernel_matrix_cdist, _, _ = PCManifoldKernel.read_kernel_output(
             kernel_output=kernel_output
         )
 
+        # choose object to copy time information from, if applicable
+        if isinstance(kernel_matrix_cdist, TSCDataFrame):
+            # if possible take time index from kernel_matrix (especially
+            # dynamics-adapted kernels can drop samples from X)
+            index_from: Optional[TSCDataFrame] = kernel_matrix_cdist
+        elif isinstance(X, TSCDataFrame) and kernel_matrix_cdist.shape[0] == X.shape[0]:
+            # if kernel is numpy.ndarray or scipy.sparse.csr_matrix, but X_fit_ is a time
+            # series, then take incides from X_fit_ -- this only works if no samples are
+            # dropped in the kernel computation.
+            index_from = X
+        else:
+            index_from = None
+
+        eigvec, eigvals = self._select_eigenpairs_target_coords()
+
         eigvec_nystroem = self._nystrom(
             kernel_matrix_cdist,
-            eigvec=np.asarray(self.eigenvectors_),
-            eigvals=self.eigenvalues_,
+            eigvec=np.asarray(eigvec),
+            eigvals=eigvals,
+            index_from=index_from,
         )
 
         return self._perform_dmap_embedding(eigvec_nystroem)
@@ -546,16 +693,20 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            See `fit` method for additional parameter.
+
         Returns
         -------
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
             same type as `X` of shape `(n_samples, n_eigenpairs)`
         """
 
-        X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_samples=2))
-        self.fit(X=X, y=y)
+        X = self._validate_datafold_data(X, array_kwargs=dict(ensure_min_samples=2))
+        self.fit(X=X, y=y, **fit_params)
 
-        return self._perform_dmap_embedding(self.eigenvectors_)
+        eigvec, _ = self._select_eigenpairs_target_coords()
+        return self._perform_dmap_embedding(eigvec)
 
     def inverse_transform(self, X: TransformType) -> TransformType:
         """Pre-image from embedding space back to original (ambient) space.
@@ -577,21 +728,29 @@ class DiffusionMaps(BaseEstimator, TSCTransformerMixin):
         """
 
         check_is_fitted(self)
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="inverse_transform")
 
         if not hasattr(self, "inv_coeff_matrix_"):
+            if isinstance(self.X_fit_, pd.DataFrame):
+                # happens if samples were dropped during kernel fit
+                _X = self.X_fit_.loc[self.eigenvectors_.index, :]
+            else:
+                _X = self.X_fit_
+
+            eigvec, _ = self._select_eigenpairs_target_coords()
+
             self.inv_coeff_matrix_ = scipy.linalg.lstsq(
-                np.asarray(self.eigenvectors_), self.X_, cond=None
+                np.asarray(eigvec), _X, cond=None
             )[0]
 
         X_orig_space = np.asarray(X) @ self.inv_coeff_matrix_
         return self._same_type_X(
-            X, values=X_orig_space, feature_names=self.features_in_.names
+            X, values=X_orig_space, feature_names=self.feature_names_in_
         )
 
 
-class DiffusionMapsVariable(TSCTransformerMixin):
+class DiffusionMapsVariable(BaseEstimator, TSCTransformerMixin):  # pragma: no cover
     """(experimental, not documented)
     .. warning::
         This class is not documented. Contributions are welcome
@@ -608,6 +767,7 @@ class DiffusionMapsVariable(TSCTransformerMixin):
     def __init__(
         self,
         epsilon=1.0,
+        *,  # keyword-only
         n_eigenpairs=10,
         nn_bandwidth=10,
         expected_dim=2,
@@ -648,18 +808,24 @@ class DiffusionMapsVariable(TSCTransformerMixin):
 
     def fit(self, X: TransformType, y=None, **fit_params):
 
-        X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_samples=2))
+        X = self._validate_datafold_data(X, array_kwargs=dict(ensure_min_samples=2))
 
-        self._setup_features_fit(
+        self._setup_feature_attrs_fit(
             X, features_out=[f"dmap{i}" for i in range(self.n_eigenpairs)]
         )
 
+        self._read_fit_params(attrs=None, fit_params=fit_params)
+
         self.dist_kwargs = self.dist_kwargs or {}
         self.dist_kwargs.setdefault("cut_off", np.inf)
         self.dist_kwargs.setdefault("kmin", self.nn_bandwidth)
         self.dist_kwargs.setdefault("backend", "guess_optimal")
 
-        pcm = PCManifold(X, kernel=self.dmap_kernel_, dist_kwargs=self.dist_kwargs,)
+        pcm = PCManifold(
+            X,
+            kernel=self.dmap_kernel_,
+            dist_kwargs=self.dist_kwargs,
+        )
 
         # basis_change_matrix is None if not required
         (
@@ -704,7 +870,7 @@ class DiffusionMapsVariable(TSCTransformerMixin):
         )
 
         self.eigenvectors_ = self._same_type_X(
-            X, values=self.eigenvectors_, feature_names=self.features_out_.names
+            X, values=self.eigenvectors_, feature_names=self.feature_names_out_
         )
 
         return self
@@ -719,7 +885,7 @@ class DiffusionMapsVariable(TSCTransformerMixin):
 
     def fit_transform(self, X: TransformType, y=None, **fit_params):
         self.fit(X=X, y=y, **fit_params)
-        return self._same_type_X(X, self.eigenvectors_, self.features_out_.names)
+        return self._same_type_X(X, self.eigenvectors_, self.feature_names_out_)
 
 
 class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
@@ -757,7 +923,7 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
 
     intrinsic_dim
         Number of eigenvectors to select with largest residuals.
-        
+
     regress_threshold
         Threshold for local residual to include eigenvectors that are above,
         if strategy="threshold".
@@ -765,6 +931,9 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
     bandwidth_type
         "median" or "mean"
 
+    random_state
+        seed for random generator if the data is subsampled
+
     Attributes
     ----------
 
@@ -784,12 +953,14 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
 
     def __init__(
         self,
+        *,  # keyword-only
         eps_med_scale=3,
         n_subsample=np.inf,
         strategy="dim",
         intrinsic_dim=2,
         regress_threshold=0.9,
         bandwidth_type="median",
+        random_state: Optional[int] = None,
     ):
 
         self.strategy = strategy
@@ -797,6 +968,7 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
         self.intrinsic_dim = intrinsic_dim
         self.regress_threshold = regress_threshold
         self.bandwidth_type = bandwidth_type
+        self.random_state = random_state
 
         self.eps_med_scale = eps_med_scale
         self.n_subsample = n_subsample
@@ -807,17 +979,18 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
             self.eps_med_scale,
             name="eps_med_scale",
             target_type=(float, np.floating, int, np.integer),
-            min_val=np.finfo(np.float64).eps,  # exclusive zero
+            min_val=np.finfo(float).eps,  # exclusive zero
             max_val=np.inf,
         )
 
-        check_scalar(
-            self.n_subsample,
-            name="n_subsample",
-            target_type=(int, np.integer),
-            min_val=100,
-            max_val=np.inf,
-        )
+        if not np.isinf(self.n_subsample):
+            check_scalar(
+                self.n_subsample,
+                name="n_subsample",
+                target_type=(int, np.integer),
+                min_val=1,
+                max_val=np.inf,
+            )
 
         if self.strategy not in self._cls_valid_strategy:
             raise ValueError(f"strategy={self.strategy} is invalid.")
@@ -836,8 +1009,8 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
                 self.regress_threshold,
                 name="regress_threshold",
                 target_type=(float, np.floating),
-                min_val=np.finfo(np.float64).eps,
-                max_val=1 - np.finfo(np.float64).eps,
+                min_val=np.finfo(float).eps,
+                max_val=1 - np.finfo(float).eps,
             )
 
         if self.bandwidth_type not in self._cls_valid_bandwidth:
@@ -874,7 +1047,7 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
             )
 
         if eps_regression == 0:
-            eps_regression = np.finfo(np.float64).eps
+            eps_regression = np.finfo(float).eps
 
         # equation 11 in referenced paper, corresponding to the weights
         kernel_eigvec = np.exp(-1 * np.square(distance_matrix_eigvec) / eps_regression)
@@ -998,6 +1171,12 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
         X
             Eigenvectors of shape `(n_samples, n_eigenvectors)` to make selection on.
 
+        y: None
+            ignored
+
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         LocalRegressionSelection
@@ -1011,13 +1190,20 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
         # Later on not all of these columns are required because of the selection
         # performed.
 
-        X = self._validate_data(X, validate_array_kwargs=dict(ensure_min_features=2))
+        X = self._validate_datafold_data(X, array_kwargs=dict(ensure_min_features=2))
         num_eigenvectors = X.shape[1]
 
         self._validate_parameter(num_eigenvectors)
 
-        if self.n_subsample is not None:
-            eigvec, _ = random_subsample(X, self.n_subsample)
+        self._read_fit_params(attrs=None, fit_params=fit_params)
+
+        if not np.isinf(self.n_subsample):
+            eigvec = resample(
+                X,
+                replace=False,
+                n_samples=self.n_subsample,
+                random_state=self.random_state,
+            )
         else:
             eigvec = np.asarray(X)
 
@@ -1033,16 +1219,12 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
 
         self._set_indices()
 
-        # Cannot use self._setup_features_fit here because the columns (if they exist)
-        # are partially selected.
-        if self._has_feature_names(X):
-            self._setup_frame_input_fit(
-                features_in=X.columns, features_out=X.columns[self.evec_indices_],
-            )
-        else:
-            self._setup_array_input_fit(
-                features_in=X.shape[1], features_out=len(self.evec_indices_)
-            )
+        self._setup_feature_attrs_fit(
+            X,
+            features_out=X.columns[self.evec_indices_]
+            if isinstance(X, pd.DataFrame)
+            else len(self.evec_indices_),
+        )
 
         return self
 
@@ -1061,14 +1243,14 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
             same type as `X` of shape `(n_samples, n_evec_indices)`
         """
 
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="transform")
 
         # choose eigenvectors
         X_selected = self._same_type_X(
             X,
             np.asarray(X)[:, self.evec_indices_],
-            feature_names=self.features_out_.names,
+            feature_names=self.feature_names_out_,
         )
 
         return X_selected
@@ -1079,13 +1261,13 @@ class LocalRegressionSelection(BaseEstimator, TSCTransformerMixin):
             Not implemented.
         """
 
-        # TODO: from the philosophy the inverse_transform should map
+        # TODO: the inverse_transform should map
         #   \Psi_selected -> \Psi_full
         #  However this is usually not what we are interested in. Instead it is more
         #  likely that someone wants to do the inverse_transform like in DMAP:
         #   \Psi_selected -> X (original data).
 
         raise NotImplementedError(
-            "The inverse_transform should be carried out with an DMAP, which has all "
-            "eigenvectors (not selected)."
+            "The inverse_transform should be carried out with an DiffusionMap model, "
+            "which contains all computed eigenvectors."
         )
