@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 import itertools
-from typing import Union
+from typing import Optional, Union
 
 import numpy as np
 import pandas as pd
@@ -32,6 +32,8 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
     """
 
     _cls_valid_scale_names = ("min-max", "standard")
+    # flag from scikit-learn -- need to set that check_estimator is valid
+    _required_parameters = ["sklearn_transformer"]
 
     def __init__(self, sklearn_transformer):
         self.sklearn_transformer = sklearn_transformer
@@ -43,6 +45,7 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
         Parameters
         ----------
         name
+            - "center" -:class:`sklearn.preprocessing.StandardScaler`
             - "min-max" - :class:`sklearn.preprocessing.MinMaxScaler`
             - "standard" - :class:`sklearn.preprocessing.StandardScaler`
 
@@ -51,6 +54,8 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
         TSCFeaturePreprocess
             new instance
         """
+        if name == "center":
+            return cls(StandardScaler(copy=True, with_mean=True, with_std=False))
         if name == "min-max":
             return cls(MinMaxScaler(feature_range=(0, 1), copy=True))
         elif name == "standard":
@@ -71,6 +76,9 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            `None`
+
         Returns
         -------
         TSCFeaturePreprocess
@@ -80,8 +88,10 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
         if not hasattr(self.sklearn_transformer, "transform"):
             raise AttributeError("sklearn object has no 'transform' attribute")
 
-        X = self._validate_data(X)
-        self._setup_features_fit(X, features_out="like_features_in")
+        X = self._validate_datafold_data(X)
+        self._setup_feature_attrs_fit(X, features_out="like_features_in")
+
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         self.sklearn_transformer_fit_ = clone(
             estimator=self.sklearn_transformer, safe=True
@@ -108,13 +118,13 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
 
         check_is_fitted(self, "sklearn_transformer_fit_")
 
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="transform")
 
         X_intern = self._X_to_numpy(X)
         values = self.sklearn_transformer_fit_.transform(X_intern)
         return self._same_type_X(
-            X=X, values=values, feature_names=self.features_out_.names
+            X=X, values=values, feature_names=self.feature_names_out_
         )
 
     def fit_transform(self, X: TransformType, y=None, **fit_params):
@@ -134,15 +144,15 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
             same type and shape as `X`
         """
 
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
 
-        self._setup_features_fit(X, features_out="like_features_in")
+        self._setup_feature_attrs_fit(X, features_out="like_features_in")
 
         self.sklearn_transformer_fit_ = clone(self.sklearn_transformer)
         values = self.sklearn_transformer_fit_.fit_transform(X)
 
         return self._same_type_X(
-            X=X, values=values, feature_names=self.features_out_.names
+            X=X, values=values, feature_names=self.feature_names_out_
         )
 
     def inverse_transform(self, X: TransformType):
@@ -165,7 +175,7 @@ class TSCFeaturePreprocess(BaseEstimator, TSCTransformerMixin):
         X_intern = self._X_to_numpy(X)
         values = self.sklearn_transformer_fit_.inverse_transform(X_intern)
         return self._same_type_X(
-            X=X, values=values, feature_names=self.features_in_.names
+            X=X, values=values, feature_names=self.feature_names_in_
         )
 
 
@@ -186,7 +196,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixin):
         True if fit has been called.
     """
 
-    def __init__(self, include_const: bool = False, rename_features: bool = False):
+    def __init__(self, *, include_const: bool = False, rename_features: bool = False):
         self.include_const = include_const
         self.rename_features = rename_features
 
@@ -201,12 +211,16 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         TSCIdentity
             self
         """
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         if self._has_feature_names(X):
             if self.rename_features:
@@ -219,7 +233,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixin):
         else:
             features_out = "like_features_in"
 
-        self._setup_features_fit(X, features_out=features_out)
+        self._setup_feature_attrs_fit(X, features_out=features_out)
 
         # Dummy attribute to indicate that fit was called
         self.is_fit_ = True
@@ -241,7 +255,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixin):
         """
         check_is_fitted(self, "is_fit_")
 
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="transform")
 
         if self._has_feature_names(X):
@@ -272,7 +286,7 @@ class TSCIdentity(BaseEstimator, TSCTransformerMixin):
             same type and shape as `X`
         """
         check_is_fitted(self, "is_fit_")
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="inverse_transform")
 
         if self.include_const:
@@ -296,26 +310,6 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
     * `PCA user guide <https://scikit-learn.org/stable/modules/decomposition.html#pca>`_
     """
 
-    def __init__(
-        self,
-        n_components=2,
-        copy=True,
-        whiten=False,
-        svd_solver="auto",
-        tol=0.0,
-        iterated_power="auto",
-        random_state=None,
-    ):
-        super(TSCPrincipalComponent, self).__init__(
-            n_components=n_components,
-            copy=copy,
-            whiten=whiten,
-            svd_solver=svd_solver,
-            tol=tol,
-            iterated_power=iterated_power,
-            random_state=random_state,
-        )
-
     def fit(self, X: TransformType, y=None, **fit_params) -> "PCA":
         """Compute the principal components from training data.
 
@@ -327,19 +321,25 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         TSCPrincipalComponent
             self
         """
 
-        X = self._validate_data(X)
-        self._setup_features_fit(
-            X, features_out=[f"pca{i}" for i in range(self.n_components)]
-        )
+        X = self._validate_datafold_data(X)
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         # validation happens here:
         super(TSCPrincipalComponent, self).fit(self._X_to_numpy(X), y=y)
+
+        self._setup_feature_attrs_fit(
+            X, features_out=[f"pca{i}" for i in range(self.n_components_)]
+        )
+
         return self
 
     def transform(self, X: TransformType):
@@ -358,12 +358,12 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
         """
 
         check_is_fitted(self)
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
 
         self._validate_feature_input(X, direction="transform")
         pca_data = super(TSCPrincipalComponent, self).transform(self._X_to_numpy(X))
         return self._same_type_X(
-            X, values=pca_data, feature_names=self.features_out_.names
+            X, values=pca_data, feature_names=self.feature_names_out_
         )
 
     def fit_transform(self, X: TransformType, y=None, **fit_params) -> TransformType:
@@ -373,7 +373,7 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
             Training data of shape `(n_samples, n_features)`.
-            
+
         y: None
             ignored
 
@@ -383,16 +383,18 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
             same type as `X` of shape `(n_samples, n_components_)`
         """
 
-        X = self._validate_data(X)
-        self._setup_features_fit(
-            X, features_out=[f"pca{i}" for i in range(self.n_components)]
-        )
+        X = self._validate_datafold_data(X)
 
         pca_values = super(TSCPrincipalComponent, self).fit_transform(
             self._X_to_numpy(X), y=y
         )
+
+        self._setup_feature_attrs_fit(
+            X, features_out=[f"pca{i}" for i in range(self.n_components_)]
+        )
+
         return self._same_type_X(
-            X, values=pca_values, feature_names=self.features_out_.names
+            X, values=pca_values, feature_names=self.feature_names_out_
         )
 
     def inverse_transform(self, X: TransformType):
@@ -416,22 +418,22 @@ class TSCPrincipalComponent(PCA, TSCTransformerMixin):
         data_orig_space = super(TSCPrincipalComponent, self).inverse_transform(X_intern)
 
         return self._same_type_X(
-            X, values=data_orig_space, feature_names=self.features_in_.names
+            X, values=data_orig_space, feature_names=self.feature_names_in_
         )
 
 
 class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
-    """Perform Takens time delay embedding on time series collection data.
+    r"""Perform Takens time delay embedding on time series collection data.
 
     Parameters
     ----------
 
-    lag
-        Number of time steps to lag before embedding starts.
-
     delays
         Number for time delays to embed.
 
+    lag
+        Number of time steps to lag before embedding starts.
+
     frequency
         Time step frequency to emebd (e.g. to embed every sample or only every second
         or third).
@@ -459,14 +461,13 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
     References
     ----------
 
-    * Original paper from Takens :cite:`rand_detecting_1981`
-    * Takens delay embedding in the context of time series data
-      :cite:`champion_discovery_2019` (the time delay embedding is then a transform
-      function of :py:class:`.EDMD` dictionary).
+    * Original paper from Takens :cite:`takens_detecting_1981`
+    * time delay embedding in the context of Koopman operator, e.g.
+      :cite:`champion_discovery_2019` or :cite:`arbabi_ergodic_2017`
     """
 
     def __init__(
-        self, lag: int = 0, delays: int = 10, frequency: int = 1, kappa: float = 0
+        self, delays: int = 10, *, lag: int = 0, frequency: int = 1, kappa: float = 0
     ):
         self.lag = lag
         self.delays = delays
@@ -476,13 +477,13 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
     def _validate_parameter(self):
 
         check_scalar(
-            self.lag, name="lag", target_type=(np.integer, int), min_val=0, max_val=None
+            self.lag, name="lag", target_type=(int, np.integer), min_val=0, max_val=None
         )
 
         check_scalar(
             self.delays,
             name="delays",
-            target_type=(np.integer, int),
+            target_type=(int, np.integer),
             min_val=1,
             max_val=None,
         )
@@ -490,7 +491,7 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         check_scalar(
             self.frequency,
             name="delays",
-            target_type=(np.integer, int),
+            target_type=(int, np.integer),
             min_val=1,
             max_val=None,
         )
@@ -498,7 +499,7 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         check_scalar(
             self.kappa,
             name="kappa",
-            target_type=(np.integer, int, np.floating, float),
+            target_type=(int, np.integer, float, np.floating),
             min_val=0.0,
             max_val=None,
         )
@@ -522,7 +523,7 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         # in case the column in not string it is important to transform it here to
         # string. Otherwise, There are mixed types (e.g. int and str), because the
         # delayed columns are all strings to indicate the delay number.
-        X.columns = X.columns.astype(np.str)
+        X.columns = X.columns.astype(np.str_)
         return X
 
     def _expand_all_delay_columns(self, cols):
@@ -540,7 +541,7 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
 
         return pd.Index(
             columns_names,
-            dtype=np.str,
+            dtype=np.str_,
             copy=False,
             name=TSCDataFrame.tsc_feature_col_name,
         )
@@ -570,13 +571,14 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
 
         """
         self._validate_parameter()
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         self.delay_indices_ = self._setup_delay_indices_array()
         self.min_timesteps_ = max(self.delay_indices_) + 1
 
-        X = self._validate_data(
+        X = self._validate_datafold_data(
             X,
-            validate_tsc_kwargs={
+            tsc_kwargs={
                 "ensure_const_delta_time": True,
                 "ensure_min_timesteps": self.min_timesteps_,
             },
@@ -590,10 +592,8 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         self.delta_time_fit_ = X.delta_time
 
         features_out = self._expand_all_delay_columns(X.columns)
+        self._setup_feature_attrs_fit(X, features_out=features_out)
 
-        self._setup_frame_input_fit(
-            features_in=X.columns, features_out=features_out,
-        )
         return self
 
     def transform(self, X: TSCDataFrame) -> TSCDataFrame:
@@ -620,9 +620,9 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
             one sample in the time delay embedding.
         """
 
-        X = self._validate_data(
+        X = self._validate_datafold_data(
             X,
-            validate_tsc_kwargs={
+            tsc_kwargs={
                 # must be same time delta as during fit
                 "ensure_delta_time": self.delta_time_fit_,
                 "ensure_min_timesteps": self.min_timesteps_,
@@ -663,6 +663,10 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         if self.kappa > 0:
             # only the delayed coordinates are multiplied with the exp factor
             kappa_vec = np.exp(-self.kappa * np.arange(1, self.delays + 1))
+
+            # the np.repeat assumes the following pattern:
+            # (a,b), (a:d1, b:d1), (a:d2, b:d2), ...
+            kappa_vec = np.repeat(kappa_vec, self.n_features_in_)
         else:
             kappa_vec = None
 
@@ -685,14 +689,14 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
             )
 
             if self.kappa > 0:
-                delayed_data = delayed_data.astype(np.float)
+                delayed_data = delayed_data.astype(float)
                 delayed_data *= kappa_vec
 
             # go back to DataFrame, and adapt the index be excluding removed indices
             df = pd.DataFrame(
                 np.hstack([original_data, delayed_data]),
                 index=df.index[max_delay:],
-                columns=self.features_out_.names,
+                columns=self.feature_names_out_,
             )
 
             delayed_timeseries[idx] = df
@@ -716,10 +720,10 @@ class TSCTakensEmbedding(BaseEstimator, TSCTransformerMixin):
         """
         check_is_fitted(self)
 
-        X = self._validate_data(X, ensure_tsc=True,)
+        X = self._validate_datafold_data(X, ensure_tsc=True)
         self._validate_feature_input(X, direction="inverse_transform")
 
-        return X.loc[:, self.features_in_.names]
+        return X.loc[:, self.feature_names_in_]
 
 
 class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
@@ -729,21 +733,23 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
     ----------
 
     kernel
-        Radial basis kernel to compute the coefficients with.
+        Radial basis kernel to compute the coefficients with. Defaults to
+        :code:`MultiquadricKernel(epsilon=1.0)`.
 
     center_type
-        Selection of what to take as centers during fit. 
+        Selection of what to take as centers during fit.
 
         * `all_data` - all data points during fit are used as centers
+        * `fit_params` - set the center points with keyword arguments during fit
         * `initial_condition` - take the initial condition states as centers.
            Note for this option the data `X` during fit must be of
            type :class:`.TSCDataFrame`.
 
     exact_distance
-        An inexact distance computation increases the performance at the cost of
+        An inexact distance computation increases computational performance at the cost of
         numerical inaccuracies (~1e-7 for Euclidean distance, and ~1 e-14 for squared
         Eucledian distance).
-    
+
     Attributes
     ----------
 
@@ -755,12 +761,13 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
         delayed until `inverse_transform` is called for the first time.
     """
 
-    _cls_valid_center_types = ["all_data", "initial_condition"]
+    _cls_valid_center_types = ["all_data", "fit_params", "initial_condition"]
 
     def __init__(
         self,
-        kernel: PCManifoldKernel = MultiquadricKernel(epsilon=1.0),
-        center_type="all_data",
+        kernel: Optional[PCManifoldKernel] = None,
+        *,  # keyword-only
+        center_type: str = "all_data",
         exact_distance=True,
     ):
         self.kernel = kernel
@@ -774,7 +781,10 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
                 f"{self._cls_valid_center_types} "
             )
 
-    def fit(self, X: TransformType, y=None, **fit_kwargs) -> "TSCRadialBasis":
+    def _get_default_kernel(self):
+        return MultiquadricKernel(epsilon=1.0)
+
+    def fit(self, X: TransformType, y=None, **fit_params) -> "TSCRadialBasis":
         """Set the point centers of the radial basis functions.
 
         Parameters
@@ -786,30 +796,66 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            centers: numpy.ndarray
+                Points where the radial basis functions are centered.
+                `center_type="fit_params"` must be set during initialization.
+
         Returns
         -------
         TSCRadialBasis
             self
         """
 
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_center_type(center_type=self.center_type)
+        _centers = self._read_fit_params(
+            attrs=[("centers", None)], fit_params=fit_params
+        )
 
         if self.center_type == "all_data":
+            if _centers is not None:
+                raise ValueError("center points were passed but center_type='all_data'")
+
             self.centers_ = self._X_to_numpy(X)
-        else:  # self.center_type == "initial_condition":
+        elif self.center_type == "fit_params":
+
+            if _centers is None:
+                raise ValueError("The center points were not provided in 'fit_params'.")
+
+            try:
+                self.centers_ = np.asarray(_centers).astype(float)
+            except TypeError:
+                raise TypeError(
+                    "centers were not passed to fit_params or not array-like."
+                )
+
+            if self.centers_.ndim != 2 or self.centers_.shape[1] != X.shape[1]:
+                raise ValueError(
+                    "The center points must be a matrix with same point "
+                    "dimension than 'X'."
+                )
+        elif self.center_type == "initial_condition":
             if not isinstance(X, TSCDataFrame):
-                raise TypeError("Data 'X' must be TSCDataFrame")
+                raise TypeError("'X' must be of type TSCDataFrame.")
             self.centers_ = X.initial_states().to_numpy()
+        else:
+            raise RuntimeError(
+                "center_type was not checked correctly. Please report bug."
+            )
+
+        set_kernel = (
+            self.kernel if self.kernel is not None else self._get_default_kernel()
+        )
 
         self.centers_ = PCManifold(
             self.centers_,
-            kernel=self.kernel,
+            kernel=set_kernel,
             dist_kwargs=dict(backend="brute", exact_numeric=self.exact_distance),
         )
 
         n_centers = self.centers_.shape[0]
-        self._setup_features_fit(X, [f"rbf{i}" for i in range(n_centers)])
+        self._setup_feature_attrs_fit(X, [f"rbf{i}" for i in range(n_centers)])
 
         return self
 
@@ -827,14 +873,14 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
             same type as `X` of shape `(n_samples, n_centers)`
         """
         check_is_fitted(self, attributes=["centers_"])
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="transform")
 
         X_intern = self._X_to_numpy(X)
         rbf_coeff = self.centers_.compute_kernel_matrix(Y=X_intern)
 
         return self._same_type_X(
-            X, values=rbf_coeff, feature_names=self.features_out_.names
+            X, values=rbf_coeff, feature_names=self.feature_names_out_
         )
 
     def fit_transform(self, X, y=None, **fit_params):
@@ -854,18 +900,19 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
         TSCDataFrame, pandas.DataFrame, numpy.ndarray
             same type as `X` of shape `(n_samples, n_centers)`
         """
-        self.fit(X)
+        self.fit(X, **fit_params)
         X_intern = self._X_to_numpy(X)
         self._validate_center_type(center_type=self.center_type)
 
         if self.center_type == "all_data":
             # compute pdist distance matrix, which is often more efficient
             rbf_coeff = self.centers_.compute_kernel_matrix()
-        else:  # self.center_type == "initial_condition":
+        else:  # self.center_type in ["initial_condition", "fit_params"]:
             rbf_coeff = self.centers_.compute_kernel_matrix(Y=X_intern)
 
+        # import matplotlib.pyplot as plt; plt.matshow(rbf_coeff)
         return self._same_type_X(
-            X=X, values=rbf_coeff, feature_names=self.features_out_.names
+            X=X, values=rbf_coeff, feature_names=self.feature_names_out_
         )
 
     def inverse_transform(self, X: TransformType):
@@ -898,7 +945,7 @@ class TSCRadialBasis(BaseEstimator, TSCTransformerMixin):
 
         X_inverse = rbf_coeff @ self.inv_coeff_matrix_
         return self._same_type_X(
-            X, values=X_inverse, feature_names=self.features_in_.names
+            X, values=X_inverse, feature_names=self.feature_names_in_
         )
 
 
@@ -917,6 +964,7 @@ class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixin):
     def __init__(
         self,
         degree: int = 2,
+        *,  # keyword-only
         interaction_only: bool = False,
         include_bias: bool = False,
         include_first_order=False,
@@ -941,7 +989,9 @@ class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixin):
     def _get_poly_feature_names(self, X, input_features=None):
         # Note: get_feature_names function is already provided by super class
         if self._has_feature_names(X):
-            feature_names = self.get_feature_names(input_features=X.columns)
+            feature_names = self.get_feature_names(
+                input_features=X.columns.astype(np.str_)
+            )
         else:
             feature_names = self.get_feature_names()
         return feature_names
@@ -958,17 +1008,25 @@ class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixin):
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
             Data of shape `(n_samples, n_features)`.
 
+        y: None
+            ignored
+
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         TSCPolynomialFeatures
             self
         """
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         super(TSCPolynomialFeatures, self).fit(X, y=y)
 
-        self._setup_features_fit(
-            X, features_out=self._get_poly_feature_names(X),
+        self._setup_feature_attrs_fit(
+            X,
+            features_out=self._get_poly_feature_names(X),
         )
 
         return self
@@ -988,7 +1046,7 @@ class TSCPolynomialFeatures(PolynomialFeatures, TSCTransformerMixin):
             as `X`.
         """
         check_is_fitted(self)
-        X = self._validate_data(X)
+        X = self._validate_datafold_data(X)
         self._validate_feature_input(X, direction="transform")
 
         poly_data = super(TSCPolynomialFeatures, self).transform(X)
@@ -1043,16 +1101,21 @@ class TSCApplyLambdas(BaseEstimator, TSCTransformerMixin):
         ----------
         X: TSCDataFrame, pandas.DataFrame, numpy.ndarray
             Training data.
+
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         TSCApplyLambdas
             self
         """
         self._not_implemented_numpy_arrays(X)
-        X = self._validate_data(X, ensure_tsc=True)
+        X = self._validate_datafold_data(X, ensure_tsc=True)
+        self._read_fit_params(attrs=None, fit_params=fit_params)
 
         features_out = [
             f"{feature_name}_lambda{i}"
@@ -1060,7 +1123,7 @@ class TSCApplyLambdas(BaseEstimator, TSCTransformerMixin):
             for i in range(len(self.lambdas))
         ]
 
-        self._setup_features_fit(X, features_out=features_out)
+        self._setup_feature_attrs_fit(X, features_out=features_out)
         return self
 
     def transform(self, X: TransformType) -> TransformType:
@@ -1080,7 +1143,7 @@ class TSCApplyLambdas(BaseEstimator, TSCTransformerMixin):
         self._not_implemented_numpy_arrays(X)
 
         check_is_fitted(self)
-        X = self._validate_data(X, ensure_tsc=True)
+        X = self._validate_datafold_data(X, ensure_tsc=True)
         self._validate_feature_input(X, direction="transform")
 
         lambdas_applied = list()
@@ -1137,7 +1200,11 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixin):
     """
 
     def __init__(
-        self, spacing: Union[str, float] = "dt", diff_order: int = 1, accuracy: int = 2
+        self,
+        *,  # keyword-only
+        spacing: Union[str, float] = "dt",
+        diff_order: int = 1,
+        accuracy: int = 2,
     ):
 
         self.spacing = spacing
@@ -1155,6 +1222,9 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixin):
         y: None
             ignored
 
+        **fit_params: Dict[str, object]
+            None
+
         Returns
         -------
         TSCFiniteDifference
@@ -1167,22 +1237,24 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixin):
             the same value as specified in `spacing` during initialization.
 
         """
-        X = self._validate_data(
+        X = self._validate_datafold_data(
             X,
             ensure_tsc=False,
-            validate_tsc_kwargs=dict(
+            tsc_kwargs=dict(
                 ensure_delta_time=self.spacing
                 if isinstance(self.spacing, float)
                 else None
             ),
         )
 
+        self._read_fit_params(attrs=None, fit_params=fit_params)
+
         if self._has_feature_names(X):
             features_out = [f"{col}_dot" for col in X.columns]
         else:
             features_out = X.shape[1]
 
-        self._setup_features_fit(X=X, features_out=features_out)
+        self._setup_feature_attrs_fit(X=X, features_out=features_out)
 
         if self.spacing == "dt":
             if not isinstance(X, TSCDataFrame):
@@ -1209,8 +1281,8 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixin):
         check_scalar(
             self.spacing_,
             "spacing",
-            target_type=(int, float, np.integer, np.floating),
-            min_val=np.finfo(np.float64).eps,
+            target_type=(int, np.integer, float, np.floating),
+            min_val=np.finfo(float).eps,
             max_val=None,
         )
         self.spacing_ = float(self.spacing_)
@@ -1252,16 +1324,19 @@ class TSCFiniteDifference(BaseEstimator, TSCTransformerMixin):
             If input `X` has a different time delta than data during `fit`.
         """
         check_is_fitted(self)
-        X = self._validate_data(
+        X = self._validate_datafold_data(
             X,
             ensure_tsc=True,
-            validate_tsc_kwargs=dict(ensure_delta_time=self.spacing_),
+            tsc_kwargs=dict(ensure_delta_time=self.spacing_),
         )
 
         self._validate_feature_input(X=X, direction="transform")
 
         time_derivative = X.tsc.time_derivative(
-            diff_order=self.diff_order, accuracy=self.accuracy
+            scheme="center",
+            diff_order=self.diff_order,
+            accuracy=self.accuracy,
+            shift_index=True,
         )
         time_derivative = time_derivative.add_suffix(f"_dot{self.diff_order}")
         return time_derivative
