@@ -10,7 +10,7 @@ from scipy.spatial.distance import cdist, pdist, squareform
 from sklearn.metrics import pairwise_distances
 from sklearn.neighbors import BallTree, NearestNeighbors
 
-from datafold.utils.general import if1dim_colvec, if1dim_rowvec
+from datafold.utils.general import if1dim_colvec, if1dim_rowvec, is_integer
 
 try:
     # rdist is an optional distance algorithm backend -- an import error is raised only
@@ -184,7 +184,7 @@ class BruteForceDist(DistanceAlgorithm):
         **backend_options,
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
         """Pair-wise distance matrix computation.
-        
+
         Parameters
         ----------
         X
@@ -202,7 +202,7 @@ class BruteForceDist(DistanceAlgorithm):
             If False, computes Euclidean distances more efficiently
             at the cost of introducing numerical noise. Empirically `~1e-14` for
             "sqeuclidean" and `~1e-7` for "euclidean" metric.
-            
+
         **backend_options
             Keyword arguments handled to the executing backend.
 
@@ -217,7 +217,9 @@ class BruteForceDist(DistanceAlgorithm):
             distance_matrix = squareform(_pdist)
         else:
             # sklearn uses an numeric inexact but faster implementation
-            distance_matrix = pairwise_distances(X, metric=self.metric)
+            distance_matrix = pairwise_distances(
+                X, metric=self.metric, **backend_options
+            )
 
         if cut_off is not None:
             distance_matrix = self._dense2csr_matrix(distance_matrix, cut_off=cut_off)
@@ -406,7 +408,7 @@ class ScipyKdTreeDist(DistanceAlgorithm):
     ----------
     metric
         "euclidean" or "sqeuclidean"
-        
+
     References
     ----------
 
@@ -668,7 +670,7 @@ class GuessOptimalDist(DistanceAlgorithm):
 
         **backend_options
             Keyword arguments passed to :meth:`DistanceAlgorithm.pdist`
-        
+
         Returns
         -------
         scipy.sparse.csr_matrix
@@ -779,7 +781,7 @@ def _ensure_kmin_nearest_neighbor(
 
         # Note: duplicates and trivial self-distances in the pdist are assumed to already
         # covered by the DistanceAlgorithm (always contained in the radius!)
-        nnz_distance_mask = (distances != 0).astype(np.bool)
+        nnz_distance_mask = (distances != 0).astype(bool)
         distances = distances[nnz_distance_mask]
 
         knn_query_indices = np.repeat(knn_query_indices, kmin)[nnz_distance_mask]
@@ -810,7 +812,7 @@ def _ensure_kmin_nearest_neighbor(
         )
 
         # TODO: This changes the sparsity structure and raises a warning. I am not sure
-        #  how to do make it right. For this attempt the tests fail:
+        #  how to make this right. For this attempt the tests fail:
         #  distance_matrix.tolil(copy=False)[
         #      kmin_elements_csr.nonzero()
         #  ] = kmin_elements_csr.data
@@ -907,7 +909,7 @@ def compute_distance_matrix(
     Y
         Reference point cloud for component-wise computation of shape \
         `(n_samples_Y, n_features_Y)`. If not given, then `Y=X` (pairwise computation)
-    
+
     metric
         Distance metric. Needs to be supported by backend.
 
@@ -920,8 +922,8 @@ def compute_distance_matrix(
             cut-off must be stated in in Eucledian distance (not squared cut-off).
 
     kmin
-        Minimum number of neighbors. Ignored if `cut_off=np.inf` to indicate a dense
-        distance matrix, where all distance pairs are computed.
+        Minimum number of neighbors per point. Ignored if `cut_off=np.inf` to indicate
+        a dense distance matrix, where all distance pairs are computed.
 
     backend
         Backend to compute distance matrix.
@@ -938,40 +940,55 @@ def compute_distance_matrix(
 
     if not isinstance(X, np.ndarray):
         X = np.asarray(X)
+
     X = if1dim_colvec(X)
 
-    if Y is not None and not isinstance(Y, np.ndarray):
+    is_pdist = Y is None
+
+    if cut_off is None or np.isinf(cut_off):
+        cut_off = None  # default to None and use dense case if np.isinf(cut_off)
+        is_sparse = False
+    else:
+        try:
+            cut_off = float(cut_off)  # make sure to only deal with Python built-in
+        except ValueError:
+            raise TypeError(f"type(cut_off)={type(cut_off)} must be of type float")
+        is_sparse = True
+
+    if not is_pdist and not isinstance(Y, np.ndarray):
         Y = np.asarray(Y)
 
-    if Y is not None:
+    if not is_pdist:
+        assert Y is not None
         Y = if1dim_rowvec(Y)
 
         if X.shape[1] != Y.shape[1]:
             raise ValueError(
-                "mismatch of point dimension: "
+                f"Mismatch in point dimension: "
                 f"X.shape[1]={X.shape[1]} != Y.shape[1]={Y.shape[1]} "
             )
 
     if X.shape[0] <= 1:
-        raise ValueError(
-            f"number of samples has to be greater than 1. Got {X.shape[0]}"
-        )
+        raise ValueError(f"Number of samples has to be greater than 1.")
+
+    if is_sparse:
+        assert isinstance(cut_off, float)
 
-    if cut_off is not None:
         if cut_off <= 0:
-            raise ValueError(f"cut_off={cut_off} must be a positive float.")
+            raise ValueError(
+                f"cut_off={cut_off} must be a positive number number of "
+                f"type 'float'"
+            )
 
-        try:
-            cut_off = float(cut_off)  # make sure to only deal with Python built-in
-        except:
-            raise TypeError(f"type(cut_off)={type(cut_off)} must be of type float")
+        kmin = int(kmin)  # use Python built-in
+        if not is_integer(kmin):
+            raise TypeError(f"kmin must be an integer type. Got: {type(kmin)}")
 
-        if np.isinf(cut_off):
-            # use dense case if cut_off is infinite
-            cut_off = None
-
-    is_pdist = Y is None
-    is_sparse = cut_off is not None
+        if kmin >= X.shape[0]:
+            raise ValueError(
+                "kmin must be smaller than the number of points available. "
+                f"Got {kmin} and X.shape[0]={X.shape[0]}"
+            )
 
     if metric == "sqeuclidean":
         if cut_off is not None:
@@ -1008,22 +1025,26 @@ def compute_distance_matrix(
         if (kmin > 0 and not is_pdist) or (kmin > 1 and is_pdist):
             # kmin == 1 and is_pdist does not need treatment because the diagonal is set.
             distance_matrix = _ensure_kmin_nearest_neighbor(
-                X, Y, metric=metric, kmin=kmin, distance_matrix=distance_matrix,
+                X,
+                Y,
+                metric=metric,
+                kmin=kmin,
+                distance_matrix=distance_matrix,
             )
 
         # sort_indices returns immediately if indices are already sorted.
         # If not sorted, the call could be costly (depending on nnz), but is better for
-        # follow-up handling.
+        # follow-up computations.
         distance_matrix.sort_indices()
 
-        n_elements_stored = (
-            distance_matrix.nnz
-            + len(distance_matrix.indptr)
-            + len(distance_matrix.indices)
-        )
+        # n_elements_stored = (
+        #     distance_matrix.nnz
+        #     + len(distance_matrix.indptr)
+        #     + len(distance_matrix.indices)
+        # )
 
         # There are also other reasons than memory savings for sparse matrices --
-        # therfore the warning is comment out for now.
+        # therefore the warning is comment out for now.
 
         # if n_elements_stored > np.product(distance_matrix.shape):
         #     warnings.warn(
