@@ -4,6 +4,7 @@ import abc
 from functools import partial
 from typing import Generator, Optional, Tuple, Union
 
+import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from sklearn import metrics
@@ -48,7 +49,7 @@ class TSCMetric(object):
     """
 
     _cls_valid_modes = ["timeseries", "timestep", "feature"]
-    _cls_valid_metrics = ["rmse", "rrmse", "mse", "mae", "max", "l2"]
+    _cls_valid_metrics = ["rmse", "rrmse", "mse", "mape", "mae", "medae", "max", "l2"]
     _cls_valid_scaling = ["id", "min-max", "standard", "l2_normalize"]
 
     def __init__(self, metric: str, mode: str, scaling: str = "id"):
@@ -67,7 +68,7 @@ class TSCMetric(object):
             self.mode = mode
         else:
             raise ValueError(
-                f"Invalid mode={mode}. Choose from {self._cls_valid_modes}"
+                f"Invalid mode='{mode}'. Choose from {self._cls_valid_modes}"
             )
 
         self.scaling = self._select_scaling(name=scaling)
@@ -118,22 +119,53 @@ class TSCMetric(object):
 
         return l2_norm
 
-    def _rmse_metric(
+    def _medae_metric(
         self, y_true, y_pred, sample_weight=None, multioutput="uniform_average"
     ):
-        # TODO: [minor] when upgrading to scikit-learn 0.22 mean_squared error has a
-        #  keyword "squared", if set to False, this computes the RMSE directly
+        """Median absolute error."""
 
-        mse_error = metrics.mean_squared_error(
-            y_true, y_pred, sample_weight=sample_weight, multioutput=multioutput
+        if sample_weight is not None:
+            raise ValueError("Median absolute error does not support sample_weight.")
+
+        return metrics.median_absolute_error(
+            y_true=y_true, y_pred=y_pred, multioutput=multioutput
         )
-        return np.sqrt(mse_error)
+
+    # def _mer_metric(
+    #     self, y_true, y_pred, sample_weight=None, multioutput="uniform_average"
+    # ):
+    #     r"""Mean error relative to mean observation
+    #     Each time series must have the same length (corresponding to a prediction
+    #     horizon).
+    #
+    #     The error is taken from https://www.ijcai.org/Proceedings/2017/0277.pdf
+    #
+    #     The MER is computed with
+    #     .. math::
+    #         \text{MER} = 100 \cdot \frac{1}{N} \sum_{i=1}^N
+    #         \frac{\vert y - \hat{y} \vert}{\bar{y}}
+    #     """
+    #     # TODO: this metric shows a problem in the current setting
+    #     #  -- it does not fir in the metric_per_[timeseries|feature|timestep]
+    #
+    #     if self.mode == "timestep":
+    #         raise ValueError("Metric 'mean error relative to mean observation' does not "
+    #                          "support mode 'timestep'.")
+    #
+    #     if sample_weight is not None:
+    #         raise NotImplementedError("Sample weight is not implemented ")
+    #
+    #     N = y_true.shape[0]
+    #     error = (100 * 1 / N * ((y_true - y_pred).abs() / y_true.mean()).sum())
+    #
+    #     if multioutput == "uniform_average":
+    #         error = np.mean(error)
+    #     return error
 
     def _rrmse_metric(
         self, y_true, y_pred, sample_weight=None, multioutput="uniform_average"
     ):
-        """Metric from :cite:`le_clainche_higher_2017`
-        """
+        """Metric from :cite:`le_clainche_higher_2017`"""
 
         if multioutput == "uniform_average":
             norm_ = np.sum(np.square(np.linalg.norm(y_true, axis=1)))
@@ -165,15 +197,21 @@ class TSCMetric(object):
     def _metric_from_str_input(self, error_metric: str):
 
         error_metric = error_metric.lower()
+        from typing import Callable
 
+        error_metric_handle: Callable
         if error_metric == "rmse":  # root mean squared error
-            error_metric_handle = self._rmse_metric
-        elif error_metric == "rrmse":
-            error_metric_handle = self._rrmse_metric
+            error_metric_handle = partial(metrics.mean_squared_error, squared=False)
+        elif error_metric == "rrmse":  # relative root mean squared error
+            error_metric_handle = self._rrmse_metric  # type: ignore
+        elif error_metric == "mape":  # mean absolute percentage error
+            error_metric_handle = metrics.mean_absolute_percentage_error  # type: ignore
         elif error_metric == "mse":
             error_metric_handle = metrics.mean_squared_error
         elif error_metric == "mae":
             error_metric_handle = metrics.mean_absolute_error
+        elif error_metric == "medae":  # median absolute error
+            error_metric_handle = self._medae_metric
         elif error_metric == "max":
             error_metric_handle = self._max_error
         elif error_metric == "l2":
@@ -231,7 +269,6 @@ class TSCMetric(object):
                 )
 
         if self._is_scalar_multioutput(multioutput=multioutput):
-
             column = self._single_column_name(multioutput=multioutput)
 
             # Make in both cases a DataFrame and later convert to Series in the scalar
@@ -241,7 +278,9 @@ class TSCMetric(object):
             )
         else:
             error_per_timeseries = pd.DataFrame(
-                np.nan, index=y_true.ids, columns=y_true.columns.to_list(),
+                np.nan,
+                index=y_true.ids,
+                columns=y_true.columns.to_list(),
             )
 
         for i, y_true_single in y_true.itertimeseries():
@@ -281,7 +320,10 @@ class TSCMetric(object):
             multioutput="raw_values",  # raw_values to tread every feature separately
         )
 
-        metric_per_feature = pd.Series(metric_per_feature, index=y_true.columns,)
+        metric_per_feature = pd.Series(
+            metric_per_feature,
+            index=y_true.columns,
+        )
         return metric_per_feature
 
     def _metric_per_timestep(
@@ -347,7 +389,7 @@ class TSCMetric(object):
         multioutput: Union[str, np.ndarray] = "raw_values",
     ) -> Union[pd.Series, pd.DataFrame]:
         """Compute metric between two time series collections.
-        
+
         Parameters
         ----------
         y_true
@@ -369,9 +411,9 @@ class TSCMetric(object):
                a different weight.
 
         multioutput
-            Handling of metric, if evaluated over multiple features (columns), specify
+            Handling of metric if evaluated over multiple features (columns). Specify
             how to weight each feature. The parameter is ignored for `mode=feature`,
-            because then each feature is a single output.
+            because each feature is a single output.
 
             * "raw_values" - returns metric per feature (i.e., the metric is not reduced)
             * "uniform_average" - returns metric averaged over all features averaged with
@@ -545,7 +587,7 @@ class TSCScoring(object):
         return factor * float(score)
 
 
-class TSCCrossValidationSplits(metaclass=abc.ABCMeta):
+class TSCCrossValidationSplit(metaclass=abc.ABCMeta):
     """Abstract base class for cross validation splits for time series data.
 
     See sub-classes for details.
@@ -560,7 +602,7 @@ class TSCCrossValidationSplits(metaclass=abc.ABCMeta):
         raise NotImplementedError("base class")
 
 
-class TSCKfoldSeries(TSCCrossValidationSplits):
+class TSCKfoldSeries(TSCCrossValidationSplit):
     """K-fold splits on entire time series.
 
     Both the training and the test set consist of time series in its original length.
@@ -657,7 +699,7 @@ class TSCKfoldSeries(TSCCrossValidationSplits):
         return self.kfold_splitter.get_n_splits(X=X, y=y, groups=groups)
 
 
-class TSCKFoldTime(TSCCrossValidationSplits):
+class TSCKFoldTime(TSCCrossValidationSplit):
     """K-fold splits on time values.
 
     The splits are along the time axis. This means that the time series collection can
@@ -735,7 +777,7 @@ class TSCKFoldTime(TSCCrossValidationSplits):
         return self.kfold_splitter.get_n_splits(X=X, y=y, groups=groups)
 
 
-class TSCWindowFoldTime(TSCCrossValidationSplits):
+class TSCWindowFoldTime(TSCCrossValidationSplit):
     """Assign windows of test samples starting from the end of the time series collection.
 
     This method is useful for time series collections with gaps (time intervals of no
@@ -856,7 +898,7 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
         X.tsc.check_const_time_delta()
 
         for test_tsc in indices_tsc.copy().tsc.iter_timevalue_window(
-            blocksize=self.test_window_length,
+            window_size=self.test_window_length,
             offset=self.test_window_length + self.test_offset,
             per_time_series=True,
         ):
@@ -934,8 +976,6 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
         Returns
         -------
         """
-        import matplotlib.pyplot as plt
-
         if test_set is not None and (
             np.isin(test_set.time_values(), X.time_values()).any()
         ):
@@ -958,13 +998,18 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
                 train_indices, test_indices, return_dropped=True
             )
 
-            ax[i].set_ylabel(f"split {i}")
-            ax[i].set_yticks([])
+            if n_splits == 1:
+                _ax = ax
+            else:
+                _ax = ax[i]
+
+            _ax.set_ylabel(f"split {i}")
+            _ax.set_yticks([])
 
             for j, (_, df) in enumerate(train_tsc.groupby(id_name)):
                 time_values = df.index.get_level_values(time_name)
                 width = time_values[-1] - time_values[0] + delta_time
-                ax[i].bar(
+                _ax.bar(
                     time_values[0],
                     height=1,
                     width=width,
@@ -976,7 +1021,7 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
             for j, (_, df) in enumerate(test_tsc.groupby(id_name)):
                 time_values = df.index.get_level_values(time_name)
                 width = time_values[-1] - time_values[0] + delta_time
-                ax[i].bar(
+                _ax.bar(
                     time_values[0],
                     height=1,
                     width=width,
@@ -988,7 +1033,7 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
             # This can be made easier when #105 is addressed
             dropped_samples = dropped_samples.index.get_level_values(time_name)
             if len(dropped_samples) > 0:
-                ax[i].bar(
+                _ax.bar(
                     dropped_samples,
                     height=1,
                     width=delta_time,
@@ -1004,7 +1049,7 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
                     time_values = df.time_values()
                     width = time_values[-1] - time_values[0]
 
-                    ax[i].bar(
+                    _ax.bar(
                         time_values[0],
                         height=1,
                         width=width,
@@ -1013,5 +1058,5 @@ class TSCWindowFoldTime(TSCCrossValidationSplits):
                         label="test" if j == 0 else None,
                     )
 
-        ax[-1].set_xlabel("time")
+        _ax.set_xlabel("time")
         plt.legend(loc="lower center")
