@@ -49,6 +49,7 @@ def is_df_same_index(
     df_right: pd.DataFrame,
     check_index=True,
     check_column=True,
+    check_names=True,
     handle="raise",
 ):
 
@@ -59,7 +60,9 @@ def is_df_same_index(
 
     if check_index:
         try:
-            pdtest.assert_index_equal(df_left.index, df_right.index, check_names=True)
+            pdtest.assert_index_equal(
+                df_left.index, df_right.index, check_names=check_names
+            )
         except AssertionError:
             if handle == "raise":
                 raise
@@ -68,7 +71,7 @@ def is_df_same_index(
     if check_column:
         try:
             pdtest.assert_index_equal(
-                df_left.columns, df_right.columns, check_names=True
+                df_left.columns, df_right.columns, check_names=check_names
             )
         except AssertionError:
             if handle == "raise":
@@ -96,7 +99,7 @@ def is_integer(n: object) -> bool:
 
     """
     return isinstance(n, (int, np.integer)) or (
-        isinstance(n, (float, np.floating)) and n.is_integer()
+        isinstance(n, (float, np.floating)) and n / int(n) == 1.0
     )
 
 
@@ -118,6 +121,23 @@ def is_float(n: object) -> bool:
     return isinstance(n, (float, np.floating))
 
 
+def is_scalar(n: object):
+    """Checks if `n` is a scalar.
+
+    Parameters
+    ----------
+    n
+        object to check
+
+    Returns
+    -------
+    bool
+        True if `n` is a scalar.
+
+    """
+    return is_float(n) or is_integer(n)
+
+
 def if1dim_colvec(vec: np.ndarray) -> np.ndarray:
     if vec.ndim == 1:
         return vec[:, np.newaxis]
@@ -125,7 +145,7 @@ def if1dim_colvec(vec: np.ndarray) -> np.ndarray:
         return vec
 
 
-def if1dim_rowvec(vec: np.ndarray):
+def if1dim_rowvec(vec: np.ndarray) -> np.ndarray:
     if vec.ndim == 1:
         return vec[np.newaxis, :]
     else:
@@ -135,7 +155,7 @@ def if1dim_rowvec(vec: np.ndarray):
 def projection_matrix_from_features(
     features_all: pd.Index, features_select: pd.Index
 ) -> scipy.sparse.csr_matrix:
-    """Compute a sparse projection matrix that maps that selects columns from a matrix.
+    r"""Compute a sparse projection matrix that maps that selects columns from a matrix.
 
     .. math::
         A \cdot P = A^*
@@ -208,7 +228,8 @@ def sort_eigenpairs(
     #  -- NOTE: the ordering according to the complex value is preferred over the
     #           ordering of absolute value because often complex conjugate eigenvalues
     #           have the same abs. value which makes sorting typically unstable (i.e.
-    #           there can be two equivalent but different according to the complex order)
+    #           there can be two equivalent abs values but different place according to
+    #           the complex order)
     idx = np.argsort(eigenvalues)
 
     if not ascending:
@@ -222,7 +243,7 @@ def sort_eigenpairs(
 def mat_dot_diagmat(
     matrix: np.ndarray, diag_elements: np.ndarray, out: Optional[np.ndarray] = None
 ) -> np.ndarray:
-    """Efficient computation of matrix time diagonal matrix.
+    """Efficient computation of "matrix times diagonal matrix".
 
     This computes
 
@@ -233,12 +254,15 @@ def mat_dot_diagmat(
 
     Parameters
     ----------
-    matrix, diag_elements
-        elements of computation
+    matrix
+        Dense matrix of shape `(I,J)`.
+
+    diag_elements
+        Diagonal elements in 1 dim. array of `J` elements.
 
     out
-        select where to write the result, Usual choice is setting it to matrix,
-        which simply overwrites matrix and is therefore more memory efficient
+        Select where to write the result. Usual choice is setting it to the full matrix
+        input for better memory efficient.
 
     Returns
     -------
@@ -248,7 +272,7 @@ def mat_dot_diagmat(
 
 
 def diagmat_dot_mat(diag_elements: np.ndarray, matrix: np.ndarray, out=None):
-    """Efficient computation of diagonal matrix times matrix.
+    """Efficient computation of "diagonal matrix times matrix".
 
     This computes
 
@@ -259,12 +283,15 @@ def diagmat_dot_mat(diag_elements: np.ndarray, matrix: np.ndarray, out=None):
 
     Parameters
     ----------
-    diag_elements, matrix,
-        elements of computation
+    diag_elements
+         Diagonal elements in 1 dim. array of `I` elements.
+
+    matrix
+        Dense matrix of shape `(I,J)`.
 
     out
-        select where to write the result, Usual choice is setting it to matrix,
-        which simply overwrites matrix and is therefore more memory efficient
+        Select where to write the result. Usual choice is setting it to the full matrix
+        input for better memory efficient.
 
     Returns
     -------
@@ -345,12 +372,17 @@ def remove_numeric_noise_symmetric_matrix(
 ) -> Union[np.ndarray, scipy.sparse.spmatrix]:
     r"""Remove numerical noise from (almost) symmetric matrix.
 
-    Noise can get sometimes be introduced in symmetric operations. The operations are
-    then exectued in a different order (e.g. due to optimizations) and can then break
-    exact symmetry.
+    Even symmetric operations can sometimes introduce noise. The
+    operations are often executed in different order, for example, evaluations such as in
 
-    This function is intended for "almost" symmetric matrices to recover symmetry.
-    Like in the following situation:
+    .. math::
+        D^{-1} M D^{-1}
+
+    where :math:`D` is a diagonal matrix. This can then break the exact floating point
+    symmetry in the matrix.
+
+    This function is intended to make recover an exact symmetry of "almost" symmetric
+    matrices, such as in the following situation:
 
     .. code::
         np.max(np.abs(matrix - matrix.T)) # 1.1102230246251565e-16
@@ -412,7 +444,11 @@ def random_subsample(
         indices in the subsample from the original array
     """
 
-    data = check_array(data, force_all_finite=False, ensure_min_samples=2,)
+    data = check_array(
+        data,
+        force_all_finite=False,
+        ensure_min_samples=2,
+    )
     assert isinstance(data, np.ndarray)  # for mypy
 
     n_samples_data = data.shape[0]
@@ -420,7 +456,7 @@ def random_subsample(
     check_scalar(
         n_samples,
         name="n_samples",
-        target_type=(int, np.integer),
+        target_type=int,
         min_val=1,
         max_val=n_samples_data - 1,
     )
